/****************************************************
 * Lüfter-Regelung:
 * Hardware: ARDUINO kompatible, Olinexino 328 mit VCC = 3,3 Volt o.ä.
 *           (oder bspw. Arduino NANO Klon mit 3,3 Volt)
 *           TFT-Display 320x200 Pixel, SPI, mit Touch (Achtung: 3,3 Volt!
 * Sensor:   TC1046 (mit linearem analogem OPutput) an A0
 * Sketch nutzt div, Libs, z.B. Adafruit ILI9341 und Adafriut GFX-Lib

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Limor Fried/Ladyada for Adafruit Industries.
  MIT license, all text above must be included in any redistribution
 ****************************************************/

// läuft auf Olimexino32u (Leonardo-Klon) / Olimexino328 (wg. 3.3 Volt) mit Adafruit IL9341-Lib
// läuft mit UNO mit Level-Shifter mit Adafruit IL9341-Lib
// Display: QVGA 2.2 TFT SPI 240x320 mit Touch-Controller

#include <EEPROM.h>
#include "SPI.h"
#include "Adafruit_GFX.h"
#include "Adafruit_ILI9341.h"
#include <TFT_Touch.h>
#include <PID_v1.h>

// TFT_LED-Pin an 3,3 Volt zur Hintergrundbeleuchtung!
#define TFT_MOSI 11
#define TFT_MISO 12
#define TFT_CLK 13

#define TFT_RST 8
#define TFT_DC 9
#define TFT_CS 10

// These are the pins used to interface between the 2046 touch controller and Arduino Pro
#define DOUT 7  /* Data out pin (T_DO) of touch screen */
#define DIN  6  /* Data in pin (T_DIN) of touch screen */
#define DCS  5  /* Chip select pin (T_CS) of touch screen */
#define DCLK 4  /* Clock pin (T_CLK) of touch screen */

// Farben Quecksilbersäule
#define mycol_blue1 0x5ADF      // dunkelblau
#define mycol_blue2 0x7BDF      
#define mycol_blue3 0x8C7F      
#define mycol_blue4 0xB5BF      // hellblau

// Farbe Buttons
#define mycol_gray1 0x5ACB          // dunkelgrau
#define mycol_gray2 0x6B6D
#define mycol_gray3 0x8C71          // bis
#define mycol_gray4 0xB5B6
#define mycol_gray5 0xDEFB          // hellgrau

/* Instance of the touch screen library */
TFT_Touch touch = TFT_Touch(DCS, DCLK, DIN, DOUT);

// Use hardware SPI (#13, #12, #11) and the above for CS/DC, without RESET (set RESET to HIGH)
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);

// If using the breakout, change pins as desired (with RESET)
//Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC, TFT_MOSI, TFT_CLK, TFT_RST, TFT_MISO);

//PID-Regler
double Setpoint, Input, Output;
PID myPID(&Input, &Output, &Setpoint,2.5,0.5,0.25, P_ON_M, REVERSE);
#define PWM 3                           // PWM-Output D3 

// Touch
uint16_t X_Coord;
uint16_t Y_Coord;
String Button1Text = "-";
String Button2Text = "+";

// Sollwert / Istwert / ADC
float soll = 0;
float ist = 0;
float Vadc = 0;

// Zeit 
unsigned long lastmillis = 0;

void setup() 
{
  Serial.begin(9600);
  Serial.println("Regelung Luefter mit TFT und Touch"); 

  tft.begin();
  delay(10);

  touch.setCal(481, 3395, 755, 3487, 320, 240, 1);
  touch.setRotation(1);
    
  // read diagnostics (optional but can help debug problems)
  uint8_t x = tft.readcommand8(ILI9341_RDMODE);
  Serial.print("Display Power Mode: 0x"); Serial.println(x, HEX);
  x = tft.readcommand8(ILI9341_RDMADCTL);
  Serial.print("MADCTL Mode: 0x"); Serial.println(x, HEX);
  x = tft.readcommand8(ILI9341_RDPIXFMT);
  Serial.print("Pixel Format: 0x"); Serial.println(x, HEX);
  x = tft.readcommand8(ILI9341_RDIMGFMT);
  Serial.print("Image Format: 0x"); Serial.println(x, HEX);
  x = tft.readcommand8(ILI9341_RDSELFDIAG);
  Serial.print("Self Diagnostic: 0x"); Serial.println(x, HEX); 
  
  Startbildschirm();

  soll = eeprom_read();                     // Sollwert-Temperatur einlesen
  if(isnan(soll)) {soll = 99;}             // if "non a number" (nan)... soll auf Startwert
  tft.setRotation(3);
  tft.fillScreen(ILI9341_BLACK);
  sollwertAnzeigen(80,90);

  tc1046Messen();                           // erste Messung ungültig
  delay(50);
  tc1046Messen();                           // nochmal
  istwertAnzeigen(80,30); 

  
  temperaturSymbol();                       // Temperatur-Symbol anzeigen
  temperaturUpdate();                       // "Quecksilbersäule" zeichnen
  
  drawButton(0, 200, 150, 40, Button1Text); // Button links
  drawButton(165, 200, 150, 40, Button2Text); // Button rechts

  // PID-Regler
  Setpoint = 10;                       // Änderungen bei Setpoint und Input wirken sich direkt auf den PID-Regler aus
  myPID.SetSampleTime(250);             // Regelungszeit in ms (default: 200 ms)
  myPID.SetMode(AUTOMATIC);             // PID einschalten

}


void loop(void) 
{
    if (touch.Pressed()) // Note this function updates coordinates stored within library variables
    {
      // Read the current X and Y axis as co-ordinates at the last touch time
      // The values were captured when Pressed() was called!
      X_Coord = touch.X();
      Y_Coord = touch.Y();
      //Serial.print(X_Coord); Serial.print(","); Serial.println(Y_Coord);
      //tft.fillCircle(X_Coord, Y_Coord, 3, ILI9341_RED);
      if(Y_Coord > 200)
      {
        if(X_Coord > 170)
        {
          soll += 0.5;
          if(soll > 99) soll = 99;
        }
        else
        {
          soll -= 0.5;
          if(soll < 0) soll = 0;
        }
        sollwertAnzeigen(80,90);
        eeprom_write(soll);                 // Sollwert in EEPROM merken
        delay(30);
      }
    }// ende touch

    tc1046Messen();                           // dauernd messen
    regeln();                                 // Regelzeit über Parameter eingestellt: 250 ms
          
    // Istwert alle 10 Sek. anzeigen, das ergibt eine ruhige Anzeige
    if (millis() > lastmillis + 10000)
    {
      istwertAnzeigen(80,30);
      temperaturUpdate();
      Serial.print("PWM-Out: "); Serial.println(Output);
      lastmillis = millis();
    }
     
}// Ende loop

//--------------------------------------------------------------------------------------
// Funktionen
//--------------------------------------------------------------------------------------

void regeln()
{
  Setpoint = map((int)soll, 0, 99, 0, 255);  
  Input = map((int) ist, 0, 99, 0, 255); 
  myPID.Compute();
  //Serial.print("PWM-Out: "); Serial.println(Output);
  // der folgende Wert ist Motorspezifisch! Pfeifen, ruckeln des Motors bei kleinem PWM-Wert vermeiden.
  // hier benutzter Motor läuft bei ca. PWM=80 an
  if(Output < 50) Output = 0;                 
  analogWrite(PWM, (int)Output);                        // PWM ausgeben an D3
}

void temperaturSymbol()                                 // Zeichnet Thermometer-Symbol
{
  tft.fillRoundRect(30,10,14,160, 3, ILI9341_RED);      // 14 = width, 160 = height
  tft.drawRoundRect(30,10,14,160, 3, ILI9341_YELLOW);   // Rand, 14 = width, 160 = height
  tft.fillCircle(37,160,25, ILI9341_RED);
  tft.drawCircle(37,160,25, ILI9341_YELLOW);
  tft.fillCircle(37,160,20, ILI9341_BLUE);
  // 3-D-Effekt
  tft.fillCircle(37,160,19, mycol_blue4);               // hellblau
  tft.fillCircle(37,160,18, mycol_blue3);               // dunkler werdend
  tft.fillCircle(37,160,17, mycol_blue2);
  tft.fillCircle(37,160,16, mycol_blue1);
}

void temperaturUpdate()
{
  // erst "Quecksilbersäule" löschen
  tft.fillRoundRect(31,11,12,130,3, ILI9341_RED);   // Rand, 12 = width, 130 = height
  
  // nun "Quecksilbersäule" neu zeichnen
  uint8_t i_ist = (int) ist;
  //map(value, fromLow, fromHigh, toLow, toHigh)
  i_ist = map(ist,0,99,140,11);               // von 0..99 Grad C zu Y= 140 zu Y=11 (i_ist ist die Y-Pos. des Säule oben)
  for(int x = 32; x < 42; x++)
  {
    tft.drawLine(x,140,x,i_ist,ILI9341_BLUE);
  }
  tft.drawLine(35,140,35,i_ist,mycol_blue1);
  tft.drawLine(36,140,36,i_ist,mycol_blue2);
  tft.drawLine(37,140,37,i_ist,mycol_blue3);
  tft.drawLine(38,140,38,i_ist,mycol_blue4);
  tft.drawLine(39,140,39,i_ist,mycol_blue3);
}


void drawButton(uint16_t x, uint16_t y, uint16_t width, uint16_t height, String& text)
{
    tft.fillRoundRect(x, y, width, height, 6, ILI9341_WHITE);
    // mit 3D-Effekt
    tft.drawRoundRect(x,y,width,height, 5, mycol_gray2);
    tft.drawRoundRect(x+1,y+1,width-2,height-2, 6, mycol_gray2);
    tft.drawRoundRect(x+2,y+2,width-3,height-3, 6, mycol_gray3);
    tft.drawRoundRect(x+3,y+3,width-6,height-6, 6, mycol_gray3);
    tft.drawRoundRect(x+4,y+4,width-8,height-8, 6, mycol_gray4);
    tft.drawRoundRect(x+5,y+5,width-10,height-10, 6, mycol_gray4);
    tft.drawRoundRect(x+6,y+6,width-12,height-12, 6, mycol_gray5);
    tft.drawRoundRect(x+7,y+7,width-14,height-14, 6, mycol_gray5);
       
    tft.setTextColor(ILI9341_BLACK, ILI9341_WHITE); // Vordergrund / Hintergrundfarbe
    tft.setTextSize(3);
    tft.setCursor(x-10+width/2, y-10+height/2);
    tft.println(text);
}

void sollwertAnzeigen(uint16_t x, uint16_t y)
{
    tft.setTextColor(ILI9341_GREEN, ILI9341_BLACK); // Vordergrund / Hintergrundfarbe
    tft.setTextSize(3);
    tft.setCursor(x,y);
    tft.print("Soll:");
    tft.setTextSize(5);
    tft.print(soll,1);                    // Anzeige 1 Stelle hinter dem Komma
}

void istwertAnzeigen(uint16_t x, uint16_t y)
{
    tft.setTextColor(ILI9341_GREEN, ILI9341_BLACK); // Vordergrund / Hintergrundfarbe
    tft.setTextSize(3);
    tft.setCursor(x,y);
    tft.print(" Ist:");
    tft.setTextSize(5);
    tft.print(ist,1);                     // Anzeige 1 Stelle hinter dem Komma
}

// temperatur in C mit TC1046 messen
void tc1046Messen()
{
  Vadc = (3.3 / 1024) * analogRead(0);
  ist = (Vadc - 0.424) / 0.00625;
}

// Zu Anfang eine Spielerei
void Startbildschirm() 
{
  tft.setRotation(3);
  tft.fillScreen(ILI9341_RED);
  delay(100);
  tft.fillScreen(ILI9341_WHITE);
  tft.setTextColor(ILI9341_RED, ILI9341_WHITE); // Vordergrund / Hintergrundfarbe
  tft.setTextSize(3);
  tft.setCursor(15,60);
  tft.println(" Luefterregelung");
  tft.println("  mit PID-Regler");
  tft.println("   by DL1DMW");
  tft.println("    - 2018 -");
  delay(3000);
  tft.fillScreen(ILI9341_BLACK);
}

// Sollwert in EEPROM speichern
void eeprom_write(float f)
{
  EEPROM.put(0, f);             // put für float, struct etc.
  delay(50);
}

float eeprom_read()
{
  float f;
  EEPROM.get(0,f);            // get für float, struct etc.
  delay(50);
  return f;
}







